package com.apobbati.samples.security;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.core.Authentication;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Map;

class SAMLPermissionEvaluator implements PermissionEvaluator {
    private Logger log = LoggerFactory.getLogger(SAMLPermissionEvaluator.class);

    private Map<String, Permission> permissionObjects;

    private AuthorizationAttributesProvider authorizationAttributesProvider;

    public AuthorizationAttributesProvider getAuthorizationAttributesProvider() {
        return authorizationAttributesProvider;
    }

    public void setAuthorizationAttributesProvider(AuthorizationAttributesProvider authorizationAttributesProvider) {
        this.authorizationAttributesProvider = authorizationAttributesProvider;
    }

    public Map<String, Permission> getPermissionObjects() {
        return permissionObjects;
    }

    public void setPermissionObjects(Map<String, Permission> permissionObjects) {
        this.permissionObjects = permissionObjects;
    }

    protected Permission lookupPermission(Authentication authentication, Object permission) {
        Permission permissionObject = null;

        if(authorizationAttributesProvider.hasAttribute(authentication, permission.toString())) {
            permissionObject = permissionObjects.get(permission);
        }

        if(permissionObject == null) {
            log.error("Permission " + permission + " not found for user!");
        }
        return permissionObject;
    }

    @Override
    @Transactional
    public boolean hasPermission(Authentication authentication, Object targetDomainObject, Object permission) {
        Permission permissionObject = lookupPermission(authentication, permission);
        if(permissionObject == null) {
            return false;
        }

        log.debug("Evaluating permission " + permission);
        return permissionObject.isAllowed(authentication, targetDomainObject);
    }

    @Override
    @Transactional
    public boolean hasPermission(Authentication authentication, Serializable targetId, String targetType, Object permission) {
        throw new UnsupportedOperationException("Not supported");
    }
}
