package com.apobbati.samples.security;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.saml.SAMLCredential;
import org.springframework.security.saml.userdetails.SAMLUserDetailsService;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

class SAMLUserDetailsServiceImpl implements SAMLUserDetailsService {
    private Logger log = LoggerFactory.getLogger(SAMLUserDetailsServiceImpl.class);

    private String uidAttribute = "EmailAddress";

    private GrantedAuthorityBuilder grantedAuthorityBuilder;

    public GrantedAuthorityBuilder getGrantedAuthorityBuilder() {
        return grantedAuthorityBuilder;
    }

    public void setGrantedAuthorityBuilder(GrantedAuthorityBuilder grantedAuthorityBuilder) {
        this.grantedAuthorityBuilder = grantedAuthorityBuilder;
    }

    public String getUidAttribute() {
        return uidAttribute;
    }

    public void setUidAttribute(String uidAttribute) {
        this.uidAttribute = uidAttribute;
    }

    public Object loadUserBySAML(SAMLCredential samlCredential) throws UsernameNotFoundException {
        log.info("----------------------------------------");
        log.info("Building UserObject from SAML Credential: " + samlCredential);

        String email = SAMLUtil.getAttributeValue(samlCredential, uidAttribute);

        if(samlCredential != null) {
            List<GrantedAuthority> grantedAuthorities = grantedAuthorityBuilder.buildAuthorities(samlCredential);

            // TODO: Make this flexible
            Map<String, String> attributes = new HashMap<String, String>();
            attributes.put("email", SAMLUtil.getAttributeValue(samlCredential, "EmailAddress"));
            attributes.put("name", SAMLUtil.getAttributeValue(samlCredential, "FullName"));
            attributes.put("uid", samlCredential.getNameID().getValue());

            return new CustomUserDetails(email, "", grantedAuthorities, attributes);
        }

        log.error("The SAMLCredential was null!");
        return null;
    }
}
