package com.apobbati.samples.security;

import org.codehaus.jackson.JsonFactory;
import org.codehaus.jackson.JsonGenerator;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.hierarchicalroles.RoleHierarchy;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;

import java.io.IOException;
import java.io.StringWriter;
import java.util.*;

public class SecurityPrincipalSerializer {
    private Logger log = LoggerFactory.getLogger(SecurityPrincipalSerializer.class);

    private ObjectMapper mapper = new ObjectMapper();

    @Autowired
    private RoleHierarchy roleHierarchy;

    public RoleHierarchy getRoleHierarchy() {
        return roleHierarchy;
    }

    public void setRoleHierarchy(RoleHierarchy roleHierarchy) {
        this.roleHierarchy = roleHierarchy;
    }

    public ObjectMapper getMapper() {
        return mapper;
    }

    public void setMapper(ObjectMapper mapper) {
        this.mapper = mapper;
    }

    public String principalToJson() {
        CustomUserDetails userDetails = (CustomUserDetails) SecurityContextHolder.getContext().getAuthentication().getPrincipal();

        Map<String, Object> map = new HashMap<String, Object>();
        List<String> accessLevels = new ArrayList<String>();

        map.put("uid", userDetails.getAttributes().get("uid"));
        map.put("name", userDetails.getAttributes().get("name"));
        map.put("email", userDetails.getAttributes().get("email"));

        Collection<? extends GrantedAuthority> authorities = roleHierarchy == null ? userDetails.getAuthorities() :
                roleHierarchy.getReachableGrantedAuthorities(userDetails.getAuthorities());
        for(GrantedAuthority grantedAuthority : authorities) {
            accessLevels.add(grantedAuthority.getAuthority());
        }
        map.put("accessLevels", accessLevels);

        try {
            StringWriter writer = new StringWriter();
            JsonGenerator jsonGenerator = new JsonFactory().createJsonGenerator(writer);
            mapper.writeValue(jsonGenerator, map);
            return writer.toString();
        }
        catch(IOException ex) {
            log.error("Failed to serialize principal to JSON!", ex);
        }
        return null;
    }
}
