package com.apobbati.samples.security;

import org.codehaus.jackson.JsonFactory;
import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.JsonParser;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.Resource;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;

import java.io.IOException;
import java.util.*;

class SimpleUserDetailsServiceImpl implements UserDetailsService {
    private Logger log = LoggerFactory.getLogger(SimpleUserDetailsServiceImpl.class);

    private Resource usersFile;

    private JsonNode users;

    public Resource getUsersFile() {
        return usersFile;
    }

    public void setUsersFile(Resource usersFile) {
        this.usersFile = usersFile;

        try {
            ObjectMapper mapper = new ObjectMapper();
            JsonParser parser = new JsonFactory().createJsonParser(usersFile.getFile());
            users = mapper.readTree(parser);
        }
        catch(IOException e) {
            log.error("Error when loading users.json!", e);
        }
    }

    @Override
    public UserDetails loadUserByUsername(String s) throws UsernameNotFoundException {
        if(users == null || !users.has(s.toLowerCase())) {
            throw new UsernameNotFoundException("Unable to find uer: " + s);
        }

        JsonNode userNode = users.get(s.toLowerCase());
        Map<String, String> attributes = new HashMap<String, String>();
        Iterator<String> it = userNode.getFieldNames();
        while(it.hasNext()) {
            String key = it.next();
            JsonNode value = userNode.get(key);
            if(key.equalsIgnoreCase("password") || value.isArray())
                continue;
            attributes.put(key, userNode.get(key).getTextValue());
        }

        JsonNode authorities = userNode.get("roles");
        List<GrantedAuthority> grantedAuthorities = new ArrayList<GrantedAuthority>();
        for(JsonNode entry : authorities) {
            grantedAuthorities.add(new SimpleGrantedAuthority(entry.getTextValue()));
        }

        log.debug("Foo=" + userNode.get("uid").getTextValue());

        return new CustomUserDetails(attributes.get("uid"), userNode.get("password").getTextValue(), grantedAuthorities, attributes);
    }
}
