'use strict';

// Declare app level module which depends on filters, and services
angular.module('myApp', ['ui.bootstrap', 'ngCookies', 'myApp.auth', 'myApp.filters', 'myApp.services', 'myApp.directives', 'myApp.controllers']).
  config(['$routeProvider', '$httpProvider', function($routeProvider, $httpProvider) {
    $routeProvider.when('/view1', {
      templateUrl: 'partials/partial1.html',
      controller: 'MyCtrl1',
      access: 'ACCESS_LEVEL1'
    });

    $routeProvider.when('/view2', {
      templateUrl: 'partials/partial2.html',
      controller: 'MyCtrl2',
      access: 'ACCESS_LEVEL2'
    });

    $routeProvider.when('/view3', {
      templateUrl: 'partials/partial3.html',
      controller: 'MyCtrl3',
      access: 'ACCESS_LEVEL3'
    });

    $routeProvider.when('/view4', {
      templateUrl: 'partials/partial4.html',
      controller: 'MyCtrl4',
      access: ['ACCESS_LEVEL2', 'ACCESS_LEVEL3']
    });

    $routeProvider.otherwise({
      redirectTo: '/view1'
    });

    var interceptor = ['$location', '$window', '$q', function($location, $window, $q) {
      function success(response) {
        return response;
      }

      function error(response) {
        if(response.status === 401 || response.status == 403) {
          // Do something else to provide feedback or a redirect
          return $q.reject(response);
        }
        else {
          return $q.reject(response);
        }
      }

      return function(promise) {
        return promise.then(success, error);
      }
    }];

    $httpProvider.responseInterceptors.push(interceptor);
  }])
  .run(['$rootScope', '$location', 'auth', function ($rootScope, $location, auth) {
    $rootScope.$on("$routeChangeStart", function (event, next, current) {
      $rootScope.error = null;
      if(next.access != null && !auth.hasAnyRole(next.access)) {
        //alert("Authorization Error: You cannot access this part of the application!");
        $location.path('/view1');
      }
    });
  }]);
